﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using System.Runtime.InteropServices.WindowsRuntime;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using WP_Afterwork.Resources;
using WP_Afterwork.Proximity;
using Microsoft.Phone.Tasks;
using System.IO;
using System.Windows.Media.Imaging;
using Nokia.Graphics.Imaging;
using Windows.Storage.Streams;

namespace WP_Afterwork
{
    public partial class MainPage : PhoneApplicationPage
    {
        #region Schritt 1 - Initialisieren

        public BluetoothConnection Connection { get; set; }

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            Connection = BluetoothConnection.Instance;
            Connection.StartAndAdvertise();
            //Wird in Schritt 3 benoetigt
            Connection.Connected = ConnectedHandler;
        }

        #endregion

        #region Schritt 2 - Wer ist noch da?

        private async void OnLookForDevices(object sender, RoutedEventArgs e)
        {
            bool hasFoundPeer = await Connection.FindPeers();
            if (hasFoundPeer)
            {
                tbResult.Text = "Ha! Gefunden :)";
                btConnect.IsEnabled = true;
            }
            else
            {
                tbResult.Text = "Nichts gefunden...";
            }
        }

        #endregion

        #region Schritt 3 - Verbinden

        private async void OnConnect(object sender, RoutedEventArgs e)
        {
            await Connection.ConnectToPeer();
        }

        public void ConnectedHandler()
        {
            Dispatcher.BeginInvoke(() =>
            {
                spSendText.Visibility = Visibility.Visible;
                btConnect.IsEnabled = false;
                btLook.IsEnabled = false;
            });
            ListenForMessages();
        }

        #endregion

        #region Schritt 4 - Senden/Empfangen

        public async void ListenForMessages()
        {
            while (true)
            {
                var response = await Connection.ListenMessage();
                if (response.Type == 1)
                {
                    Dispatcher.BeginInvoke(() => tbText.Text = (string)response.Data);
                }
                else if (response.Type == 2)
                {
                    //using System.Runtime.InteropServices.WindowsRuntime; !!
                    IBuffer buffer = ((byte[])response.Data).AsBuffer();
                    Session = new EditingSession(buffer);
                    await Session.RenderToImageAsync(imgTest);
                }
                
            }
        }

        private void OnSendText(object sender, RoutedEventArgs e)
        {
            Connection.SendMessage(tbText.Text);
        }

        #endregion

        //Imaging SDK

        #region Schritt 1 - Init

        public EditingSession Session { get; set; }

        // - Build-Target auf ARM umstellen
        // - SDK zum Projekt hinzufuegen

        #endregion

        #region Schritt 2 - Bild waehlen

        private void OnChoosePic(object sender, RoutedEventArgs e)
        {
            PhotoChooserTask pct = new PhotoChooserTask();
            pct.Completed += pct_Completed;
            pct.Show();
        }

        async void pct_Completed(object sender, PhotoResult e)
        {
            if (e.TaskResult == TaskResult.OK)
            {
                Session = await EditingSessionFactory.CreateEditingSessionAsync(e.ChosenPhoto);

                await Session.RenderToImageAsync(imgTest);
                spActions.Visibility = Visibility.Visible;
            }
        }

        #endregion

        #region Schritt 3 - Manipulationen

        private async void OnAddFilter(object sender, RoutedEventArgs e)
        {
            IFilter filter = Nokia.Graphics.Imaging.FilterFactory.CreateOilyFilter();
            Session.AddFilter(filter);

            await Session.RenderToImageAsync(imgTest);

        }

        private async void OnRotate(object sender, RoutedEventArgs e)
        {
            Session.AddFilter(FilterFactory.CreateFreeRotationFilter(180, RotationResizeMode.Ignore));
            await Session.RenderToImageAsync(imgTest);
        }

        private async void OnUndo(object sender, RoutedEventArgs e)
        {
            if (Session.CanUndo())
            {
                Session.Undo();
                await Session.RenderToImageAsync(imgTest);
            }
            else
            {
                MessageBox.Show("Nichts mehr da!");
            }
        }

        #endregion

        #region Schritt 4 (Zugabe) - Senden

        private async void OnSendPicture(object sender, RoutedEventArgs e)
        {
            if (Connection.Peer == null)
            {
                MessageBox.Show("Nicht verbunden!");
                return;
            }

            IBuffer buffer = await Session.RenderToJpegAsync();
            Connection.SendImage(buffer.ToArray());

        }

        #endregion


    }
}