﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Networking.Proximity;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;

namespace WP_Afterwork.Proximity
{
    public class BluetoothConnection
    {
        #region Singleton
        
        private static BluetoothConnection _instance;
        public static BluetoothConnection Instance
        {
            get
            {
                if (_instance == null) _instance = new BluetoothConnection();
                return _instance;
            }
        }

        private BluetoothConnection() { }

        #endregion

        #region Schritt 1 - Ich bin da

        public void StartAndAdvertise()
        {
            PeerFinder.Start();
            PeerFinder.ConnectionRequested += PeerFinder_ConnectionRequested;
        }

        #endregion

        #region Schritt 2 - Wer ist noch da?

        public PeerInformation Peer { get; set; }

        public async Task<bool> FindPeers()
        {
            int loops = 0;
            while (loops < 20)
            {
                var allPeers = await PeerFinder.FindAllPeersAsync();
                if (allPeers.Count > 0)
                {
                    Peer = allPeers[0];
                    break;
                }
                loops++;
            }
            return Peer != null;
        }

        #endregion

        #region Schritt 3 - Verbinden

        public StreamSocket Socket { get; set; }
        public DataReader Reader { get; set; }
        public DataWriter Writer { get; set; }
        public Action Connected { get; set; }

        public async Task ConnectToPeer()
        {
            Socket = await PeerFinder.ConnectAsync(Peer);
            Reader = new DataReader(Socket.InputStream);
            Writer = new DataWriter(Socket.OutputStream);
            if (Connected != null) Connected();
        }

        private async void PeerFinder_ConnectionRequested(object sender, ConnectionRequestedEventArgs args)
        {
            Peer = args.PeerInformation;
            await ConnectToPeer();
        }

        #endregion

        #region Schritt 4 - Senden/Empfangen

        public async void SendMessage(string msg)
        {
            //Protokoll
            //4 Bytes: type
            //4 Bytes: length = X
            //X Bytes: string

            //Wird spaeter benoetigt
            Writer.WriteInt32(1);

            Writer.WriteInt32(msg.Length);
            Writer.WriteString(msg);
            await Writer.StoreAsync();
        }

        public class Response
        {
            public object Data { get; set; }
            public int Type { get; set; }
        }

        public async Task<Response> ListenMessage()
        {
            var response = new Response();

            //"Header" des Protokolls (8 Byte) laden
            await Reader.LoadAsync(8);
            response.Type = Reader.ReadInt32();
            uint length = (uint)Reader.ReadInt32();

            //Daten laden
            var res = await Reader.LoadAsync(length);

            if (response.Type==1)
            {
                //Strings auslesen
                response.Data = Reader.ReadString(length);
            }
            else if (response.Type==2)
            {
                //Bytes (Bilder) auslesen (fuer spaeter)
                var bytes = new byte[length];
                Reader.ReadBytes(bytes);
                response.Data = bytes;
            }

            return response;
        }

        #endregion


        #region Bild senden (Zugabe)

        public async void SendImage(byte[] img)
        {
            Writer.WriteInt32(2);
            Writer.WriteInt32(img.Length);
            Writer.WriteBytes(img);
            await Writer.StoreAsync();
        }


        #endregion

    }
}
